/*
        Zzap Thermo v1.0
    Made by Bits from Italy
  ...'cause we love to invent
  https://www.bitsfromitaly.it

Zzap Thermo is an application for the control of thermal plants through
the measurement of different values.
The following measurements are saved:
- system pressure
- environmental temperature
- fluid delivery temperature
- flue gas temperature
- value of the system supply voltage
- presence of fume refluxes (CO measurement)
All data is saved via Thing Speak and can be viewed from the ThingSpeak
website or via the app for Android freely downloadable from Play Store.

This work has been released under a
Creative Commons Attribution-Non-Commercial-No Derivative Works 4.0 International license.
To read a copy of the license, visit the website http://creativecommons.org/licenses/by-nc-nd/4.0/.
*/
#include "ThingSpeak.h"
#include <WiFi.h>                           // manage wifi network
#include <OneWire.h>                        // manage One Wire probes
#include <DallasTemperature.h>              // manage temperature probes
#include <Wire.h>                           // smoke probe management
#include <LiquidCrystal_I2C.h>              // manage LCD
#include <max6675.h>
#include "secrets.h"                        // file containing user variables
#include "string.h"
// #include "EmonLib.h"                        // voltage reading module management

// internet connection control variable

bool netConnected = false;

// settings for voltage measurement

// EnergyMonitor emon1;
// int i = 0;
// float tenvals = 0.0;
// float minval = 1000;
// float maxval = 0.0;

// settaggi per la definizione degli allarmi - settings for defining alarms

String var_impianto = IMPIANTO;
String allarme_1 = ALLARME1;
String allarme_2 = ALLARME2;
String allarme_3 = ALLARME3;
String allarme_4 = ALLARME4;
String allarme_5 = ALLARME5;
String allarme_6 = ALLARME6;
String allarme_7 = ALLARME7;
String allarme_8 = ALLARME8;
String allarme_9 = ALLARME9;
String allarme_10 = ALLARME10;
String temp_allarme_1;
String temp_allarme_2;
String temp_allarme_3;
String temp_allarme_4;
String temp_allarme_5;
String temp_allarme_6;
String temp_allarme_7;
String temp_allarme_8;
String temp_allarme_9;
int var_allarme_1;  // minimum pressure
int var_allarme_2;  // maximum pressure
int var_allarme_3;  // minimum flow temperature
int var_allarme_4;  // maximum flow temperature
int var_allarme_5;  // minimum flue gas temperature
int var_allarme_6;  // maximum flue gas temperature
int var_allarme_7;  // CO threshold
int var_allarme_8;  // minimum voltage level
int var_allarme_9;  // maximum voltage level

String allarmeInviato = "null";  // variable to save which alarm was sent

// settaggi per la definizione del pulsante di riconoscimento d'allarme
// settings for defining the alarm acknowledgment button

const byte interruptPin = 34;

// settaggi per la definizione delle sonde - settings for the definition of the probes

int thermoDO = 19;
int thermoCS = 23;
int thermoCLK = 5;
MAX6675 thermocouple(thermoCLK, thermoCS, thermoDO);

#define ONE_WIRE_BUS (32)                             // temperature probe pins
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
DeviceAddress sensor2 = INDIRIZZO_PRIMA_SONDA;        // probe definition 1
DeviceAddress sensor3 = INDIRIZZO_SECONDA_SONDA;      // probe definition 2
DeviceAddress sensor4 = INDIRIZZO_TERZA_SONDA;        // probe definition 3

const int COsensor = 14;                              // CO sensor pin
int valoreCOmisurato;

// settings for voltage reading
// const int Vsensor = 26;                               // voltage sensor pin

// settings for pressure reading

const int Psensor = 27;

// settaggi per la definizione del monitor LCD - settings for the definition of LCD screen

LiquidCrystal_I2C lcd(0x27, 20, 4);

byte segnaleWIFI[] = {
  B00001,
  B00001,
  B00101,
  B00101,
  B00101,
  B10101,
  B10101,
  B10101
};

byte pericolo[] = {
  B01110,
  B01110,
  B01110,
  B01110,
  B01110,
  B00000,
  B01110,
  B01110
};

// settaggi per il servizio SMTP - settings for SMTP service

char server[] = "mail.smtp2go.com";
int port = 2525;
int ack = 0;
String mitt = MITTENTE;
String dest = DESTINATARIO;

// settaggi per il wifi - wifi settings

const char* ssid     = STASSID;
const char* password = STAPSK;
WiFiClient client;

// settaggi per ThingSpeak - Thingspeak settings

unsigned long myChannelNumber = SECRET_CH_ID;
const char * myWriteAPIKey = SECRET_WRITE_APIKEY;
const char * myReadAPIKey = SECRET_READ_APIKEY;
unsigned int alarmsFieldNumber = 8;

// settaggi per le letture - read settings

int number1 = 1;
int number2 = 1;
int number3 = 1;
int number4 = 1;
int number5 = 220;
int number6 = 1;
int number7 = 20;

// --------------------------------------- SETUP ----------------------------------------

void setup() {
  Serial.begin(115200); // Avvio seriale - start serial line

  lcd.begin(21, 22);                                                                // Inizializza lo schermo - start LCD
  lcd.backlight();                                                                  // Illumina lo schermo - turn on LCD illumination
  lcd.createChar(0, segnaleWIFI);                                                   // creazione icona segnale WIFI - creation of WIFI signal icon
  lcd.createChar(1, pericolo);                                                      // creazione icona di allarme inviato - creation of alarm icon sent
  
  pinMode(interruptPin, INPUT_PULLUP);                                              // pulsante - acknowledge button
  attachInterrupt(digitalPinToInterrupt(interruptPin), pulsantePremuto, FALLING);

  // emon1.voltage(26, 115, 1.7);                                                    // Tensione: pin del sensore, calibrazione - Voltage: sensor pin, calibration

  pinMode(COsensor, INPUT);                                                         // sensore CO - CO sensor

  pinMode(Psensor, INPUT);                                                          // sensore di pressione - pressure sensor
  
  Serial.println();                                                                 // connessione alla rete wifi - wifi connection
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  if (WiFi.status() == WL_CONNECTED) {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    lcd.setCursor(0,1);
    lcd.print("Connected to WIFI");
    lcd.setCursor(0,2);
    lcd.print(ssid);
    delay(4000);
    lcd.clear();
    netConnected = true;
  } else {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    lcd.setCursor(0,0);
    lcd.print("Error");
    lcd.setCursor(0,1);
    lcd.print("connecting");
    lcd.setCursor(0,2);
    lcd.print("to the WIFI");
    lcd.setCursor(0,3);
    lcd.print(ssid);
    delay(4000);
    lcd.clear();
    netConnected = false;
  }
  delay(1000);                             // Tempo necessario alla rete WiFi per connettersi - give Wifi some time to complete the connection

  sensors.begin();

  delay(1000);
  ThingSpeak.begin(client);                // inizializza ThingSpeak - start ThingSpeak service

  lcd.print("Zzap Suite - Thermo");
  lcd.setCursor(4,2);
  lcd.print("version 1.0");
  delay(3000);
  lcd.clear();
}

// ---------------------------------------- LOOP ----------------------------------------

void loop() {

  int statusCode = 0;                                             //verifica iniziale della presenza di un allarme in atto - initial verification of the presence of an alarm in progress

    // Lettura dal canale di Thingspeak delle impostazioni di allarme - Reading the alarm settings from the Thingspeak channel
  
  String alarmsList = ThingSpeak.readStringField(myChannelNumber, alarmsFieldNumber, myReadAPIKey);
  delay(500);
  
  statusCode = ThingSpeak.getLastReadStatus();                    //verifica dei dati raccolti - verification of the data collected
  if(statusCode == 200){
    Serial.println("OK");
  }
  else{
    Serial.println("Problem reading channel. HTTP error code " + String(statusCode)); 
  }
  temp_allarme_1 = alarmsList.substring(0, 3);
  var_allarme_1 = temp_allarme_1.toInt();
  temp_allarme_2 = alarmsList.substring(4, 7);
  var_allarme_2 = temp_allarme_2.toInt();
  temp_allarme_3 = alarmsList.substring(8, 11);
  var_allarme_3 = temp_allarme_3.toInt();
  temp_allarme_4 = alarmsList.substring(12, 15);
  var_allarme_4 = temp_allarme_4.toInt();
  temp_allarme_5 = alarmsList.substring(16, 19);
  var_allarme_5 = temp_allarme_5.toInt();
  temp_allarme_6 = alarmsList.substring(20, 23);
  var_allarme_6 = temp_allarme_6.toInt();
  temp_allarme_7 = alarmsList.substring(24, 27);
  var_allarme_7 = temp_allarme_7.toInt();
  temp_allarme_8 = alarmsList.substring(28, 31);
  var_allarme_8 = temp_allarme_8.toInt();
  temp_allarme_9 = alarmsList.substring(32, 35);
  var_allarme_9 = temp_allarme_9.toInt();

  //raccolta dei dati di temperatura - collection of temperature data

  sensors.requestTemperatures();            // Invio del comando per ricevere le temperature - Send the command to receive the temperatures
  Serial.println(ack);
  if (ack==0) {                             // se allarmi assenti o tacitati - in case of no alarms
      lcd.clear();                          // ripulisco il monitor - clear LCD monitor
      lcd.setCursor(0,0);                   // posiziono il cursore sulla prima colonna, prima riga - I place the cursor on the first column, first row
      lcd.print("Pressure: ");
      lcd.setCursor(13,0);
      // Misura pressione - Pressure (PSI) = ( Analog Reading - 102 ) * maxPSI /  ( 921 - 102 )
      // number1 = (analogRead(Psensor) - 102) * 58 / 614;
      // Serial.println(number1);
      lcd.print(number1);                   // scrivo il valore di pressione - write pressure value
      if (netConnected) {
        lcd.setCursor(19,0);
        lcd.write(0);
      } else {
        lcd.setCursor(19,0);
        lcd.print("");
      }
      lcd.setCursor(0,1);                   // prima colonna, seconda riga
      lcd.print("Outdoor temp: ");
      lcd.setCursor(13,1);
      Serial.print("The first probe measures: ");
      Serial.println(sensors.getTempC(sensor2));
      lcd.print(sensors.getTempC(sensor2)); // scrivo il valore della sonda - write ambient temperature value
      lcd.setCursor(0,2);                   // prima colonna, terza riga
      lcd.print("Flow temp: ");          
      lcd.setCursor(13,2);
      Serial.print("The second probe measures: ");
      Serial.println(sensors.getTempC(sensor3));
      lcd.print(sensors.getTempC(sensor3)); // scrivo il valore della sonda - write flow temperature value
      lcd.setCursor(0,3);                   // prima colonna, terza riga
      lcd.print("Temp smoke: ");
      lcd.setCursor(13,3);
      Serial.print("The smoke probe measures: ");
      Serial.println(thermocouple.readCelsius());
      lcd.print(thermocouple.readCelsius()); // scrivo il valore della sonda - write gas temperature value
      delay(6000);
      lcd.clear();                          // ripulisco il monitor - clear LCD monitor
      lcd.setCursor(0,0);                   // posiziono il cursore sulla prima colonna, prima riga
      lcd.print("Voltage: ");
      lcd.setCursor(13,0);
      lcd.print(number5);                   // scrivo il valore di tensione - write pressure value
      lcd.setCursor(16,0);
      lcd.print("V");
      if (netConnected) {
        lcd.setCursor(19,0);
        lcd.write(0);
      } else {
        lcd.setCursor(19,0);
        lcd.print("");
      }
      lcd.setCursor(0,1);                   // prima colonna, seconda riga
      lcd.print("CO sensor: ");
      lcd.setCursor(13,1);
      Serial.print("The CO probe measures: ");
      Serial.println(number6);
      lcd.print(number6); // scrivo il valore della sonda CO - write ambient temperature value
  } else {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.write(1);
    lcd.setCursor(1,0);
    lcd.print("Alarm sent");
    lcd.setCursor(1,1);
    lcd.print(allarmeInviato);
    lcd.setCursor(1,2);
    lcd.print("press RESET");
    lcd.setCursor(1,3);
    lcd.print("to continue");
    delay(2000);
  }
  
  // compilo i campi da spedire a ThingSpeak con i valori raccolti - assign collected values to Thingspeak fields
  ThingSpeak.setField(1, number1);
  ThingSpeak.setField(2, number2);
  ThingSpeak.setField(3, number3);
  ThingSpeak.setField(4, number4);
  ThingSpeak.setField(5, number5);
  ThingSpeak.setField(6, number6);
  ThingSpeak.setField(7, number7);
  ThingSpeak.setField(8, alarmsList);

  // Invio dei dati a Thingspeak - send data to Thingspeak
  int x = ThingSpeak.writeFields(myChannelNumber, myWriteAPIKey);
  if(x == 200){
    Serial.println("Channel update successful.");
  }
  else{
    Serial.println("Problem updating channel. HTTP error code " + String(x));
  }

  // Aggiornamento dei dati - refresh data
 
  number1 = 2;                                  // pressione
  number2 = sensors.getTempC(sensor2);          // temperatura esterna
  number3 = sensors.getTempC(sensor3);          // temperatura in mandata
  number4 = thermocouple.readCelsius();         // temperatura fumi
  number5 = 220 ;                               // valore di tensione
  valoreCOmisurato = (analogRead(COsensor)/1024)*5.0;
  number6 = valoreCOmisurato;                   // valore rilevatore CO
  number7 = sensors.getTempC(sensor4);          // temperatura ambiente
  
  
  // verifica presenza allarmi - check alarms
  if (allarmeInviato == "null") {
      //-------------- pressione impianto ---------
      if (number1 < var_allarme_1){
        sendEmail(var_impianto, number1, allarme_1);
        allarmeInviato = allarme_1;
      }
      if (number1 > var_allarme_2){
        sendEmail(var_impianto, number1, allarme_2);
        allarmeInviato = allarme_2;
      }
      //-------------temperatura esterna-----------
      if (number2 < 5){
        sendEmail(var_impianto, number2, allarme_10);
        allarmeInviato = allarme_10;
      }
      //-----------temperatura mandata-------------
      if (number3 < 15){
        sendEmail(var_impianto, number3, allarme_3);
        allarmeInviato = allarme_3;
      }
      if (number3 > 35){
        sendEmail(var_impianto, number3, allarme_4);
        allarmeInviato = allarme_4;
      }
      //------------temperatura fumi---------------
      if (number4 < var_allarme_5){
        sendEmail(var_impianto, number4, allarme_5);
        allarmeInviato = allarme_5;
      }
      if (number4 > var_allarme_6){
        sendEmail(var_impianto, number4, allarme_6);
        allarmeInviato = allarme_6;
      }
      //-----------tensione alimentazione----------
      if (number5 < var_allarme_8){
        sendEmail(var_impianto, number5, allarme_7);
        allarmeInviato = allarme_7;
      }
      if (number5 > var_allarme_9){
        sendEmail(var_impianto, number5, allarme_8);
        allarmeInviato = allarme_8;
      }
      //----------------Sensore CO-----------------
      if (number6 != 1){
        sendEmail(var_impianto, number6, allarme_9);
        allarmeInviato = allarme_9;
      }
  }
  delay(30000);  // attendo 5 minuti - I wait 5 minutes
}

void pulsantePremuto() {
  ack = 0;
  allarmeInviato = "null";
  Serial.print("Button pressed and alarm set to: ");
  Serial.println(allarmeInviato);
}

void sendEmail(String nome_impianto, int sonda, String tipo_allarme)
  {
  byte thisByte = 0;
  byte respCode;
  if(client.connect(server,port) == 1) {
  Serial.println(F("connected"));
  } else {
  Serial.println(F("connection failed"));
  return;
  }
  
  if(!eRcv()) return;
  
  client.println("EHLO 151.95.140.110");             // inserisci l'ip della scheda Wemos  - ip board
  if(!eRcv()) return;
  
  client.println("auth login");
  if(!eRcv()) return;
  
  client.println(F("xxxxxxxxxxxxxxxxxxxxxxxxxxxxx")); // inserire lo username codificato in base64 - encode your email username in base64
  
  if(!eRcv()) return;

  client.println(F("XXXXXXXXXX="));                 // inserire la password codificata in base64 - encode your email password in base64
  
  
  if(!eRcv()) return;
  
  client.println("MAIL From: "+mitt); // indirizzo email del mittente - sender email address
  if(!eRcv()) return;
  
  client.println("RCPT To: "+dest); // indirizzo email del destinatario - recipient email address
  if(!eRcv()) return;
  
  Serial.println(F("Sending DATA"));
  client.println("DATA");
  if(!eRcv()) return;
  
  client.println("To: "+dest);   // indirizzo email del destinatario - recipient email address
  
  client.println("From: "+mitt);   // indirizzo email del mittente - sender email address
  
  client.println("Subject: Warning message from "+String(nome_impianto));   // Email subject with the name of the system that generates the alarm
  
  client.println("This is an alert message: "+String(tipo_allarme)); // Message body with the type of alarm generated
  
  client.println(".");
  
  if(!eRcv()) return;
  
  client.println("QUIT");
  if(!eRcv()) return;
  
  client.stop();
  lcd.clear();                    // ripulisco lo schermo per pubblicare il messaggio di allarme - clean monitor to write alarm message
  lcd.setCursor(0,1);
  lcd.print("Alarm sent");
  lcd.setCursor(0,2);
  lcd.print(tipo_allarme);
  Serial.println("Email sent");
  ack = 1;                        // variabile acknowledge a 1 per non perdere la schermata di allerta, dovrai premere il pulsante di acquisizione per tornare a vedere i valori
  return;
}

byte eRcv()     // funzione che controlla il time-out del client - timeout check function
  {
  byte respCode;
  byte thisByte;
  int loopCount = 0;
  while(!client.available()) {
    delay(1);
    loopCount++;
    if(loopCount > 10000) {       // se non ricevi nulla entro 10 secondi vai in time-out
        client.stop();
        Serial.println(F("\r\nTimeout"));
        return 0;
      }
  }
  respCode = client.peek();
  while(client.available())
    {
      thisByte = client.read();
      Serial.write(thisByte);
    }
  if(respCode >= '4')
    {
      efail();
      return 0;
    }
  return 1;
}

void efail()    // funzione che controlla il time-out della connessione al server di posta - timeout server connection
  {
  byte thisByte = 0;
  int loopCount = 0;
  client.println(F("QUIT"));
  while(!client.available()) {
    delay(1);
    loopCount++;
    if(loopCount > 10000) {       // se non viene ricevuto nulla entro 10 secondi vai in time-out
      client.stop();
      Serial.println(F("\r\nTimeout"));
      return;
      }
  }
  while(client.available())
  {
    thisByte = client.read();
    Serial.write(thisByte);
  }
  client.stop();
  Serial.println(F("disconnected"));
}
